/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.apio.architect.test.util.form;

import com.liferay.apio.architect.form.Form;
import com.liferay.apio.architect.form.Form.Builder;
import com.liferay.apio.architect.impl.internal.form.FormImpl.BuilderImpl;

import java.util.Arrays;
import java.util.function.BiConsumer;

/**
 * Provides methods that create {@link Form} objects.
 *
 * <p>
 * This class shouldn't be instantiated.
 * </p>
 *
 * @author Alejandro Hernández
 */
public class MockFormCreator {

	/**
	 * Creates a mock {@code Form}.
	 *
	 * @param  paths the list of paths for the form
	 * @return the mock {@code Form}
	 */
	public static Form createForm(String... paths) {
		Builder<Object> builder = new BuilderImpl<>(
			Arrays.asList(paths), __ -> null);

		return builder.title(
			__ -> "title"
		).description(
			__ -> "description"
		).constructor(
			Object::new
		).addOptionalBoolean(
			"boolean1", _emptyBiConsumer()
		).addOptionalDate(
			"date1", _emptyBiConsumer()
		).addOptionalDouble(
			"double1", _emptyBiConsumer()
		).addOptionalLong(
			"long1", _emptyBiConsumer()
		).addOptionalString(
			"string1", _emptyBiConsumer()
		).addRequiredBoolean(
			"boolean2", _emptyBiConsumer()
		).addRequiredDate(
			"date2", _emptyBiConsumer()
		).addRequiredDouble(
			"double2", _emptyBiConsumer()
		).addRequiredLong(
			"long2", _emptyBiConsumer()
		).addRequiredString(
			"string2", _emptyBiConsumer()
		).build();
	}

	private static <T> BiConsumer<Object, T> _emptyBiConsumer() {
		return (object, t) -> {
		};
	}

	private MockFormCreator() {
		throw new UnsupportedOperationException();
	}

}