/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.apio.architect.test.util.internal.json;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;

import com.liferay.apio.architect.test.util.json.AbstractJsonElementMatcher;
import com.liferay.apio.architect.test.util.json.JsonElementType;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;

import org.hamcrest.Matcher;

/**
 * A {@code Matcher} that checks if an element is a correct JSON array.
 *
 * @author Alejandro Hernández
 */
public class IsJsonArray
	extends AbstractJsonElementMatcher
		<JsonArray, Iterable<? extends JsonElement>> {

	public IsJsonArray(
		Matcher<Iterable<? extends JsonElement>> jsonArrayMatcher) {

		super(
			JsonElementType.ARRAY, " that is an ", jsonArrayMatcher,
			jsonArray -> {
				List<JsonElement> jsonElements = new ArrayList<>();

				Iterator<JsonElement> iterator = jsonArray.iterator();

				iterator.forEachRemaining(jsonElements::add);

				return jsonElements;
			});

		Objects.requireNonNull(jsonArrayMatcher);
	}

}