/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.apio.architect.test.util.internal.json;

import com.google.gson.Gson;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;

import com.liferay.apio.architect.functional.Try;

import java.util.Objects;

import org.hamcrest.Description;
import org.hamcrest.Matcher;
import org.hamcrest.TypeSafeDiagnosingMatcher;

/**
 * A {@link Matcher} that checks if a string is a correct JSON object.
 *
 * @author Alejandro Hernández
 */
public class IsJsonObjectString extends TypeSafeDiagnosingMatcher<String> {

	public IsJsonObjectString(Matcher<JsonElement> jsonObjectMatcher) {
		_jsonObjectMatcher = jsonObjectMatcher;
	}

	@Override
	public void describeTo(Description description) {
		description.appendDescriptionOf(_jsonObjectMatcher);
	}

	@Override
	protected boolean matchesSafely(String item, Description description) {
		return Try.fromFallible(
			() -> new Gson().fromJson(item, JsonObject.class)
		).filter(
			Objects::nonNull
		).fold(
			__ -> {
				description.appendText("was not a JSON object");

				return false;
			},
			jsonObject -> {
				if (_jsonObjectMatcher.matches(jsonObject)) {
					return true;
				}
				else {
					_jsonObjectMatcher.describeMismatch(
						jsonObject, description);

					return false;
				}
			}
		);
	}

	private final Matcher<JsonElement> _jsonObjectMatcher;

}