/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.apio.architect.test.util.internal.list;

import com.liferay.apio.architect.impl.internal.list.FunctionalList;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.hamcrest.Description;
import org.hamcrest.Matcher;
import org.hamcrest.TypeSafeDiagnosingMatcher;

/**
 * A {@code Matcher} that checks if an element is a {@link FunctionalList}
 * containing some values.
 *
 * @author Alejandro Hernández
 * @param  <E> the type of elements in the list
 */
public class IsAFunctionalList<E>
	extends TypeSafeDiagnosingMatcher<FunctionalList<E>> {

	public IsAFunctionalList(Matcher<Iterable<? extends E>> matcher) {
		_matcher = matcher;
	}

	@Override
	public void describeTo(final Description description) {
		description.appendText(
			"a functional list that conforms with an "
		).appendDescriptionOf(
			_matcher
		);
	}

	@Override
	protected boolean matchesSafely(
		final FunctionalList<E> functionalList, final Description description) {

		E head = functionalList.head();

		List<E> list = Stream.concat(
			Stream.of(head), functionalList.tailStream()
		).collect(
			Collectors.toList()
		);

		if (_matcher.matches(list)) {
			return true;
		}

		description.appendText("was a functional list whose ");

		_matcher.describeMismatch(list, description);

		return false;
	}

	private final Matcher<Iterable<? extends E>> _matcher;

}