/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.apio.architect.test.util.internal.result;

import com.liferay.apio.architect.functional.Try;

import org.hamcrest.Description;
import org.hamcrest.TypeSafeDiagnosingMatcher;

/**
 * Provides a {@code org.hamcrest.Matcher} that checks if an element is a {@link
 * Try.Failure}.
 *
 * @author Alejandro Hernández
 * @param  <T> the {@code Try}'s possible value type
 */
public class FailTry<T> extends TypeSafeDiagnosingMatcher<Try<T>> {

	@Override
	public void describeTo(final Description description) {
		description.appendText("a Failure");
	}

	@Override
	protected boolean matchesSafely(
		final Try<T> tTry, final Description description) {

		if (tTry.isFailure()) {
			return true;
		}

		description.appendText("was a Success");

		return false;
	}

}