/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.apio.architect.test.util.internal.util;

import java.util.function.Consumer;

import org.hamcrest.Description;
import org.hamcrest.StringDescription;

/**
 * Provides utility functions hamcrest {@code Description}.
 *
 * <p>
 * This class shouldn't be instantiated.
 * </p>
 *
 * @author Alejandro Hernández
 */
public class DescriptionUtil {

	/**
	 * Appends and indents a field's description inside the provided
	 * description.
	 *
	 * @param description the provided description
	 * @param key the field's key
	 * @param consumer the consumer that updates the field's description
	 */
	public static void indentDescription(
		Description description, String key, Consumer<Description> consumer) {

		Description stringDescription = new StringDescription();

		consumer.accept(stringDescription);

		String string = stringDescription.toString();

		String trimmed = string.trim();

		description.appendText(
			"  " + key + ": "
		).appendText(
			trimmed.replaceAll("\n", "\n  ")
		).appendText(
			"\n"
		);
	}

	private DescriptionUtil() {
		throw new UnsupportedOperationException();
	}

}