/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.apio.architect.test.util.internal.writer;

import static com.liferay.apio.architect.test.util.writer.MockWriterUtil.getRequestInfo;

import com.google.gson.Gson;
import com.google.gson.JsonObject;

import com.liferay.apio.architect.impl.internal.message.json.FormMessageMapper;
import com.liferay.apio.architect.impl.internal.request.RequestInfo;
import com.liferay.apio.architect.impl.internal.writer.FormWriter;
import com.liferay.apio.architect.test.util.form.MockFormCreator;

import javax.ws.rs.core.HttpHeaders;

/**
 * Provides methods that test {@link FormMessageMapper} objects.
 *
 * <p>
 * This class shouldn't be instantiated.
 * </p>
 *
 * @author Alejandro Hernández
 */
public class MockFormWriter {

	/**
	 * Writes a {@link com.liferay.apio.architect.form.Form} object.
	 *
	 * @param httpHeaders the request's HTTP headers
	 * @param formMessageMapper the {@code FormMessageMapper} to use for writing
	 *        the JSON object
	 */
	public static JsonObject write(
		HttpHeaders httpHeaders, FormMessageMapper formMessageMapper) {

		RequestInfo requestInfo = getRequestInfo(httpHeaders);

		FormWriter formWriter = FormWriter.create(
			builder -> builder.form(
				MockFormCreator.createForm("f", "s")
			).formMessageMapper(
				formMessageMapper
			).requestInfo(
				requestInfo
			).build());

		return new Gson().fromJson(formWriter.write(), JsonObject.class);
	}

	private MockFormWriter() {
		throw new UnsupportedOperationException();
	}

}