/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.apio.architect.test.util.list;

import com.liferay.apio.architect.impl.internal.list.FunctionalList;
import com.liferay.apio.architect.test.util.internal.list.IsAFunctionalList;

import org.hamcrest.Matcher;

/**
 * Provides {@code Matcher} objects that can be used for testing a {@link
 * FunctionalList}.
 *
 * <p>
 * This class shouldn't be instantiated.
 * </p>
 *
 * @author Alejandro Hernández
 */
public final class FunctionalListMatchers {

	/**
	 * Returns a matcher that checks if this is a functional list that conforms
	 * with the {@code matcher} parameter.
	 *
	 * @param  matcher the matcher used to check the contents of this functional
	 *         interface
	 * @return the matcher that checks if this is a functional list that
	 *         conforms with the {@code matcher} parameter
	 */
	public static <T> Matcher<FunctionalList<T>> aFunctionalListThat(
		Matcher<Iterable<? extends T>> matcher) {

		return new IsAFunctionalList<>(matcher);
	}

	private FunctionalListMatchers() {
		throw new UnsupportedOperationException();
	}

}