/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.apio.architect.test.util.result;

import com.liferay.apio.architect.functional.Try;
import com.liferay.apio.architect.test.util.internal.result.FailTry;
import com.liferay.apio.architect.test.util.internal.result.SuccessTry;
import com.liferay.apio.architect.test.util.internal.result.ValueTry;

import org.hamcrest.Matcher;

/**
 * Provides {@code Matcher} objects that can be used for testing the {@link Try}
 * monadic type.
 *
 * <p>
 * This class shouldn't be instantiated.
 * </p>
 *
 * @author Alejandro Hernández
 */
public final class TryMatchers {

	/**
	 * Returns a matcher that checks if this {@code Try} is a {@code
	 * Try.Failure}.
	 *
	 * @return the matcher that checks if this {@code Try} is a failure
	 */
	public static <T> Matcher<Try<T>> aFailTry() {
		return new FailTry<>();
	}

	/**
	 * Returns a matcher that checks if this {@code Try} is a {@code
	 * Try.Success}.
	 *
	 * @return the matcher that checks if this try is a success
	 */
	public static <T> Matcher<Try<T>> aSuccessTry() {
		return new SuccessTry<>();
	}

	/**
	 * Returns a matcher that checks if this {@code Try} is a {@code
	 * Try.Success} with a certain value.
	 *
	 * @param  matcher the matcher used to check the value
	 * @return the matcher that checks if this {@code Try} is a success with a
	 *         certain value
	 */
	public static <T> Matcher<Try<T>> aTryWithValueThat(
		final Matcher<T> matcher) {

		return new ValueTry<>(matcher);
	}

	private TryMatchers() {
		throw new UnsupportedOperationException();
	}

}